<?php

if ( ! class_exists( 'SLPlus_AdminUI_GeneralSettings' ) ) {

    /**
     * Store Locator Plus General Settings Interface
     *
     * @package   StoreLocatorPlus\AdminUI\GeneralSettings
     * @author    Lance Cleveland <lance@charlestonsw.com>
     * @copyright 2013 - 2016 Charleston Software Associates, LLC
     *
     * @property-read        SLP_Settings $settings
     */
    class SLPlus_AdminUI_GeneralSettings extends SLPlus_BaseClass_Object {
        private $settings;

        /**
         * Build this thing.
         */
        function initialize() {
            $this->create_object_settings();
        }

        /**
         * General / Data
         *
         * @param   SLP_Settings    $settings
         */
        public function add_data_subtab( $settings ) {
            $section_params['name'] = __( 'Data' , 'store-locator-le' );
            $section_params['slug'] = 'data';
            $settings->add_section( $section_params );

            $group_params['header']       = __( 'Packaged Data Extensions' , 'store-locator-le' );
            $group_params['group_slug']   = 'packaged_data_extensions';
            $group_params['section_slug'] = $section_params['slug'];
            $group_params['intro'       ] = __( 'Add pre-defined groups of extended data fields to your locations. ' , 'store-locator-le' );
            $group_params['plugin'      ] = $this->slplus;
            $settings->add_group( $group_params );

            // Don't show the notice if these add-ons are active.
            //
            if ( $this->slplus->add_ons->is_active( 'slp-premier' )  ) { return; }
            if ( $this->slplus->add_ons->is_active( 'slp-power'   )  ) { return; }

            // Note that some add-on packs will put settings here.
            //
            $settings->add_ItemToGroup(array(
                'group_params'  => $group_params,
                'type'          => 'details'    ,
                'custom'        =>
                    sprintf( __( 'Some of the %s add ons will add settings here. ' , 'store-locator-le' ) ,  $this->slplus->text_manager->get_text_string('slp')  ) .
                    sprintf( __( 'Visit the %s to see a full list of features you can add to this plugin. ' , 'store-locator-le' ) , $this->slplus->get_web_link('slp_store') )
            ));
        }

        /**
         * Add Premium Subscription section to settings page.
         *
         * @param array $group_params
         */
        function add_premium_subscription_settings( $group_params ) {

            $this->settings->add_ItemToGroup(array(
                    'group_params' => $group_params,
                    'label'       => $this->slplus->text_manager->get_admin_text( 'premium_members' ),
                    'type'        => 'subheader',
                    'description' =>
                        $this->slplus->text_manager->get_linked_text( 'premium_member_updates' ) . '<br/>' .
                        $this->slplus->text_manager->get_admin_text( 'premium_member_support' )
            ));

            $this->settings->add_ItemToGroup(array(
                    'group_params' => $group_params,
                    'option_name' => 'options_nojs',
                    'option'      => 'premium_user_id',
                    'label'       => $this->slplus->text_manager->get_admin_text( 'premium_user_id_label' ),
                    'description' => $this->slplus->text_manager->get_admin_text( 'premium_user_id_help' )
            ));

            $this->settings->add_ItemToGroup(array(
                    'group_params' => $group_params,
                    'option_name' => 'options_nojs',
                    'option'      => 'premium_subscription_id',
                    'label'       => $this->slplus->text_manager->get_admin_text( 'premium_subscription_id_label' ),
                    'description' => $this->slplus->text_manager->get_admin_text( 'premium_subscription_id_help' )
            ));
        }

        /**
         * Create and attach settings.
         */
        function create_object_settings() {
            if ( ! isset( $this->settings ) ) {
                require_once( 'class.settings.php' );
                $this->settings = new SLP_Settings( array(
                    'name'        => $this->slplus->name . ' - ' . __( 'General Settings', 'store-locator-le' ),
                    'form_action' => admin_url() . 'admin.php?page=slp_general'
                ) );
            }
        }

        /**
         * Execute the save settings action.
         */
        function save_options() {
            if ( ! isset( $_REQUEST ) ) { return; }
            /**
             * ACTION: slp_save_generalsettings
             *
             * Runs before saving the general settings data.
             *
             * @param   SLP_Settings $settings
             */
            do_action( 'slp_save_generalsettings' );

            // Checkboxes
            // FILTER: slp_save_general_settings_checkboxes
            //
            $BoxesToHit = apply_filters( 'slp_save_general_settings_checkboxes', array() );
            foreach ( $BoxesToHit as $JustAnotherBox ) {
                $this->slplus->helper->SaveCheckboxToDB( $JustAnotherBox, '', '' );
            }

            // Serialized Checkboxes, Need To Blank If Not Received
            //
            $BoxesToHit = array(
                'extended_admin_messages',
                'force_load_js',
                'no_google_js'
            );
            foreach ( $BoxesToHit as $BoxName ) {
                if ( ! isset( $_REQUEST[ $BoxName ] ) ) {
                    $_REQUEST[ $BoxName ] = '0';
                }
            }

            // Serialized Options Setting for stuff going into slp.js.
            // This should be used for ALL new JavaScript options.
            //
            array_walk( $_REQUEST, array( $this->slplus, 'set_ValidOptions' ) );
            if ( isset( $_REQUEST['options'] ) ) {
                array_walk( $_REQUEST['options'], array( $this->slplus, 'set_ValidOptionsNoJS' ) );
            }
            update_option( SLPLUS_PREFIX . '-options', $this->slplus->options );

            // Serialized Options Setting for stuff NOT going to slp.js.
            // This should be used for ALL new options not going to slp.js.
            //
            array_walk( $_REQUEST, array( $this->slplus, 'set_ValidOptionsNoJS' ) );
            if ( isset( $_REQUEST['options_nojs'] ) ) {
                array_walk( $_REQUEST['options_nojs'], array( $this->slplus, 'set_ValidOptionsNoJS' ) );
            }
            update_option( SLPLUS_PREFIX . '-options_nojs', $this->slplus->options_nojs );
        }

        /**
         * Build the admin settings panel.
         */
        function build_AdminSettingsPanel() {
            $section_params['slug'] = 'admin';
            $section_params['name'] = __( 'Admin' , 'store-locator-le' );
            $this->settings->add_section( $section_params );

            // Store Locator Plus
            //
            $group_params['header']         = __( 'Store Locator Plus ', 'store-locator-le' );
            $group_params['group_slug']     = 'store_locator_plus';
            $group_params['section_slug']   = $section_params['slug'];
            $group_params['plugin']         = $this->slplus;
            $this->settings->add_group( $group_params );

            $this->settings->add_ItemToGroup(array(
                'group_params'  => $group_params,
                'option_name'   => 'options_nojs',
                'option'        => 'admin_locations_per_page',
                'type'          => 'hyperbutton',
                'button_label'  => __('Reset Manage Locations' , 'store-locator-le'),
                'value'         => '10',
                'description'   => __('Reset Locations Manage tab to show 10 locations at a time.' , 'store-locator-le' ),
                'onClick'       => 'SLP_ADMIN.options.change_option(this); alert("'.__('Manage Locations set to show 10 locations.' , 'store-locator-le').'"); return "10"; '
            ));

            // Add On Packs
            //
            $group_params['header']         = __( 'Add On Packs ', 'store-locator-le' );
            $group_params['group_slug']     = 'add_on_packs';
            $group_params['section_slug']   = $section_params['slug'];
            $group_params['plugin']         = $this->slplus;
            $this->settings->add_group( $group_params );

            $this->add_premium_subscription_settings( $group_params );

            $this->settings->add_ItemToGroup(array(
                'group_params' => $group_params,
                'label'        => __('Update Notices' , 'store-locator-le'),
                'type'         => 'subheader',
                'description'  => __('Leave this set to production unless you are running a staging site and wish to try the latest features or test an upcoming release.' , 'store-locator-le' )
            ));

            $this->settings->add_ItemToGroup( array(
                'group_params'  => $group_params,
                'option_name'   => 'options_nojs',
                'option'        => 'build_target',
                'label'         => __( 'SLP Plugin Updates', 'store-locator-le' ),
                'type'          => 'dropdown',
                'custom'        => array(
                    array( 'label' => 'production' ),
                    array( 'label' => 'prerelease' )
                ),
                'description' =>
                    __( 'Which build target do you want to update your  plugins to? ', 'store-locator-le' ) .
                    __( 'Leave this as production unless you really know what you are doing! ', 'store-locator-le' )
            ) );
            /**
             * ACTION: slp_generalsettings_modify_adminpanel
             *
             * Modify the General / Admin Subtab
             *
             * @param   SLP_Settings $settings
             * @param   string $section_name
             * @param   array $section_params ['name'] = string of name, ['slug'] = string of unique slug
             */
            do_action( 'slp_generalsettings_modify_adminpanel', $this->settings, $section_params['name'], $section_params );
        }

        /**
         * Build the Google settings panel.
         */
        function build_ServerSection() {
            $section_params['slug'] = 'server';
            $section_params['name'] = __( 'Server' , 'store-locator-le' );
            $this->settings->add_section( $section_params );

            $group_params['header']         = __( 'Geocoding', 'store-locator-le' );
            $group_params['group_slug']     = 'geocoding';
            $group_params['section_slug']   = $section_params['slug'];
            $group_params['plugin']         = $this->slplus;
            $this->settings->add_group( $group_params );

            $this->settings->add_ItemToGroup(array(
                    'group_params'  => $group_params,
                    'option_name'   => 'options_nojs',
                    'option'        => 'http_timeout',
                    'type'          => 'list',
                    'custom'        => array(
                        'Slow'   => '30',
                        'Normal' => '10',
                        'Fast'   => '3',
                        ),
                    'label'         => __( 'Server-To-Server Speed', 'store-locator-le' ),
                    'description' =>
                        __( 'How fast is your server when communicating with other servers like Google? ', 'store-locator-le' ) .
                        __( 'Set this to slow if you get frequent geocoding errors but geocoding works sometimes. ', 'store-locator-le' ) .
                        __( 'Set this to fast if you never have geocoding errors and are bulk loading more than 100 locations at a time. ', 'store-locator-le' )
            ));

            $this->settings->add_ItemToGroup(array(
                'group_params'  => $group_params,
                'option_name'   => 'options_nojs',
                'option'        => 'geocode_retries',
                'type'          => 'list',
                'custom'        => array(
                    'None' => 0,
                    '1'    => '1',
                    '2'    => '2',
                    '3'    => '3',
                    '4'    => '4',
                    '5'    => '5',
                    '6'    => '6',
                    '7'    => '7',
                    ),
                'label'         => __( 'Geocode Retries', 'store-locator-le' ),
                'description'   =>
                    __( 'How many times should we try to set the latitude/longitude for a new address? ', 'store-locator-le' ) .
                    __( 'Higher numbers mean slower bulk uploads. ', 'store-locator-le' ) .
                    __( 'Lower numbers make it more likely the location will not be set during bulk uploads. ', 'store-locator-le' )
            ) );

            $this->settings->add_ItemToGroup(array(
                    'group_params'  => $group_params,
                    'option_name'   => 'options_nojs',
                    'option'        => 'retry_maximum_delay',
                    'label'         => __( 'Maximum Retry Delay', 'store-locator-le' ),
                    'description'   => __( 'Maximum time to wait between retries, in seconds. ', 'store-locator-le' ) .
                                       __( 'Use multiples of 1. ', 'store-locator-le' ) .
                                       __( 'Recommended value is 5. ', 'store-locator-le' )
            ) );

            // Google License
            //
            $group_params['header']         = __( 'Google Business License', 'store-locator-le' );
            $group_params['group_slug']     = 'google_business_license';
            $group_params['section_slug']   = $section_params['slug'];
            $group_params['plugin']         = $this->slplus;
            $this->settings->add_group( $group_params );

            $this->settings->add_ItemToGroup(array(
                    'group_params'  => $group_params,
                    'option_name'   => 'options_nojs',
                    'option'        => 'google_client_id',
                    'label'         => __( 'Google Client ID', 'store-locator-le' ),
                    'description'   => __( 'If you have a Google Maps for Work client ID, enter it here. ', 'store-locator-le' ) .
                                       __( 'All Google API requests will go through your account at Google. ', 'store-locator-le' ) .
                                       __( 'You will receive higher quotas and faster maps I/O performance. ', 'store-locator-le' )
            ));

            $this->settings->add_ItemToGroup(array(
                    'group_params'  => $group_params,
                    'option_name'   => 'options_nojs',
                    'option'        => 'google_private_key',
                    'label'         => __( 'Google Private Key', 'store-locator-le' ),
                    'description'   => __( 'Your Google private key (Crypto Key) for signing Geocoding requests. ', 'store-locator-le' ) .
                                       __( 'Do NOT share this with anyone and take extra measures to keep it private. ', 'store-locator-le' )
            ));

            // Google Developers API
            //
            $group_params['header']         = __( 'Google Developers Console', 'store-locator-le' );
            $group_params['group_slug']     = 'google_developers_console';
            $group_params['section_slug']   = $section_params['slug'];
            $group_params['plugin']         = $this->slplus;
            $this->settings->add_group( $group_params );

            $this->settings->add_ItemToGroup(array(
                    'group_params'  => $group_params,
                    'option_name'   => 'options_nojs',
                    'option'        => 'google_server_key',
                    'label'         => __( 'Google API Server Key', 'store-locator-le' ),
                    'description'   => sprintf( __( 'If you have created a %s, enter it here. ', 'store-locator-le' ) , $this->slplus->get_web_link( 'google_api_key' ) ) .
                                       __( 'You will not gain performance benefits from using a Google API Server Key. ', 'store-locator-le' ) .
                                       __( 'You do have the option to setup pay-as-you-go at Google and get higher geocoding quotas. ', 'store-locator-le' )
            ));
        }

        /**
         * Build the User Settings Panel
         */
        function build_UserSettingsPanel() {
            $section_params['name'] = __( 'User Interface', 'store-locator-le' );
            $section_params['slug'] = 'user_interface';
            $this->settings->add_section( $section_params );

            // Group: JavaScript / javascript
            //
            $group_params['section_slug'] = $section_params['slug'];
            $group_params['group_slug']   = 'javascript';
            $group_params['header']       = __( 'JavaScript', 'store-locator-le' );
            $group_params['intro']        = __( 'These settings impact how JavaScript works on your site. ', 'store-locator-le' );
            $group_params['plugin']       = $this->slplus;
            if ( $this->slplus->is_CheckTrue( $this->slplus->options_nojs['force_load_js'] ) ) {
                $group_params['intro'] .=
                    '<br/>' .
                    sprintf(
                        __( '<a href="%s" target="csa">Force Load JavaScript</a> is ON. ', 'store-locator-le' ),
                        $this->slplus->support_url . '/blog/category/general/'
                    ) .
                    __( 'You are running a less-capable version Store Locator Plus. ', 'store-locator-le' );
            }

            $this->settings->add_group( $group_params );

            $this->settings->add_ItemToGroup( array(
                'group_params' => $group_params,
                'option_name'  => 'options_nojs',
                'option'       => 'ui_jquery_version',
                'disabled'     => true,
                'label'        => __( 'UI jQuery Version', 'store-locator-le' ),
                'description'  => __( 'This is the version of jQuery running on your UI.', 'store-locator-le' ) 
            ) );
            

            $this->settings->add_ItemToGroup( array(
                'group_params' => $group_params,
                'option_name'  => 'options_nojs',
                'option'       => 'force_load_js',
                'type'         => 'checkbox',
                'label'        => __( 'Force Load JavaScript', 'store-locator-le' ),
                'description'  =>
                    __( 'Force the JavaScript for Store Locator Plus to load in the header of EVERY page on your site. ', 'store-locator-le' ) .
                    __( 'This can slow down your site and will disable various Store Locator Plus features. ', 'store-locator-le' ) .
                    __( 'If you need to do this to make SLP work you should ask your theme author to add proper wp_footer() support to their code. ', 'store-locator-le' )
            ) );

            $this->settings->add_ItemToGroup( array(
                'group_params' => $group_params,
                'option_name'  => 'options_nojs',
                'option'       => 'no_google_js',
                'type'         => 'checkbox',
                'label'        => __( 'Turn Off SLP Maps', 'store-locator-le' ),
                'description'  => __( 'Check this box if your Theme or another plugin is providing Google Maps and generating warning messages. ', 'store-locator-le' ) .
                                  __( 'THIS MAY BREAK THIS PLUGIN.', 'store-locator-le' )
            ) );

            /**
             * ACTION: slp_generalsettings_modify_userpanel
             *
             * Modify the General / User Interface Subtab
             *
             * @param   SLP_Settings $settings
             * @param   string $section_name
             * @param   array $section_params ['name'] = string of name, ['slug'] = string of unique slug
             */
            do_action( 'slp_generalsettings_modify_userpanel', $this->settings, $section_params['name'], $section_params );
        }

        /**
         * Add web app settings.
         */
        function build_WebAppSettings() {
            $section_params['slug'] = 'server';
            $section_params['name'] = __( 'Server' , 'store-locator-le' );
            $this->settings->add_section( $section_params );

            $group_params['header']         = __( 'Web App Settings', 'store-locator-le' );
            $group_params['group_slug']     = 'web_app_settings';
            $group_params['section_slug']   = $section_params['slug'];
            $group_params['plugin']         = $this->slplus;
            $this->settings->add_group( $group_params );

            $this->settings->add_ItemToGroup(array(
                    'group_params' => $group_params,
                    'option_name' => 'options_nojs',
                    'option'      => 'php_max_execution_time',
                    'label'       => __( 'PHP Time Limit', 'store-locator-le' ),
                    'description' =>
                        __( 'Maximum execution time, in seconds, for PHP processing. ', 'store-locator-le' ) .
                        __( 'Affects all CSV imports for add-ons and Janitor delete all locations. ', 'store-locator-le' ) .
                        __( 'SLP Default 600. ', 'store-locator-le' ) .
                        __( 'On most servers you will need to edit this setting in the php.ini file. ', 'store-locator-le' ) .
                        sprintf( __( 'Your server default %s. ', 'store-locator-le' ), ini_get( 'max_execution_time' )
                        )
            ));
        }

        /**
         * Render the map settings admin page.
         */
        function render_adminpage() {
            $section_params['slug'       ] = 'navigation';
            $section_params['name'       ] = __( 'Navigation' , 'store-locator-le' );
            $section_params['description'] = $this->slplus->AdminUI->create_Navbar();
            $section_params['div_id'     ] = 'navbar_wrapper';
            $section_params['innerdiv'   ] = false;
            $section_params['is_topmenu' ] = true;
            $section_params['auto'       ] = false;

            $this->settings->add_section( $section_params );

            add_action( 'slp_build_general_settings_panels', array( $this, 'build_AdminSettingsPanel'   ), 10 );
            add_action( 'slp_build_general_settings_panels', array( $this, 'build_UserSettingsPanel'    ), 20 );
            add_action( 'slp_build_general_settings_panels', array( $this, 'build_ServerSection'        ), 30 );
            add_action( 'slp_build_general_settings_panels', array( $this, 'build_WebAppSettings'       ), 40 );
            add_action( 'slp_build_general_settings_panels', array( $this , 'add_data_subtab'           ), 50 );

            /**
             * ACTION: slp_build_general_settings_panels
             *
             * Modify the General Tab
             *
             * @param   SLP_Settings $settings
             */
            do_action( 'slp_build_general_settings_panels', $this->settings );

            $this->settings->render_settings_page();
        }
    }
}
